'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _JSON$stringify = _interopDefault(require('@babel/runtime/core-js/json/stringify'));
var rollupPluginutils = require('rollup-pluginutils');
var _regeneratorRuntime = _interopDefault(require('@babel/runtime/regenerator'));
var _asyncToGenerator = _interopDefault(require('@babel/runtime/helpers/asyncToGenerator'));
var componentCompiler = require('@vue/component-compiler');
var path = require('path');
var _WeakMap = _interopDefault(require('@babel/runtime/core-js/weak-map'));
var _Set = _interopDefault(require('@babel/runtime/core-js/set'));
var _objectSpread = _interopDefault(require('@babel/runtime/helpers/objectSpread'));
var componentCompilerUtils = require('@vue/component-compiler-utils');
var hash = _interopDefault(require('hash-sum'));

function createVueFilter(include, exclude) {
  var filter = rollupPluginutils.createFilter(include, exclude);
  return function (id) {
    return filter(id) && id.endsWith('.vue');
  };
}
var REGEX = /\.vue\?{[^}]+}#\.[a-z]+$/i;
function isVuePartRequest(id) {
  return REGEX.test(id);
}
function createVuePartRequest(filename, lang, type, index) {
  return filename + '?' + _JSON$stringify({
    type: type,
    index: index
  }) + '#.' + (lang || createVuePartRequest.defaultLang[type]);
}
createVuePartRequest.defaultLang = {
  template: 'html',
  styles: 'css',
  script: 'js'
  /**
   * @export
   * @param {string} id
   * @returns {{filename: string, meta: { type: string, index?: number }} | undefined}
   */

};
function parseVuePartRequest(id) {
  if (!id.includes('.vue')) return;
  var length = id.indexOf('.vue') + 4;
  var filename = id.substr(0, length);
  var query = id.substr(length + 1).replace(/#.[a-z]+$/, '');

  try {
    var meta = JSON.parse(query);
    return {
      filename: filename,
      meta: meta
    };
  } catch (e) {}
}
function resolveVuePart(descriptors, id) {
  var descriptor = descriptors[id.filename];
  var part = descriptor[id.meta.type];
  return Array.isArray(part) ? part[id.meta.index] : part;
}

function vue() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var isVue = createVueFilter(opts.include, opts.exclude);
  delete opts.include;
  delete opts.exclude;
  var compiler = componentCompiler.createDefaultCompiler(opts);
  return {
    name: 'vue',
    transform: function () {
      var _transform = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee(source, filename) {
        var descriptor, result;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (isVue(filename)) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt("return");

              case 2:
                descriptor = compiler.compileToDescriptor(filename, source);

                if (descriptor.template && descriptor.template.errors && descriptor.template.errors.length) {
                  console.error('> Errors: ' + path.relative(process.cwd(), filename) + '\n' + descriptor.template.errors.map(function (it) {
                    return '  - ' + it;
                  }).join('\n'));
                }

                if (descriptor.template && descriptor.template.tips && descriptor.template.tips.length) {
                  console.log('> Tips: ' + path.relative(process.cwd(), filename) + '\n' + descriptor.template.tips.map(function (it) {
                    return '  - ' + it;
                  }).join('\n'));
                }

                result = componentCompiler.assemble(compiler, filename, descriptor);
                return _context.abrupt("return", result.code);

              case 7:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function transform(_x, _x2) {
        return _transform.apply(this, arguments);
      };
    }()
  };
}

function vue$1() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var isVue = createVueFilter(opts.include, opts.exclude);
  var isProduction = process.env.NODE_ENV === 'production';
  var compiler = componentCompiler.createDefaultCompiler(opts.compiler);
  createVuePartRequest.defaultLang = _objectSpread({}, createVuePartRequest.defaultLang, opts.defaultLang);
  var blacklisted = new _Set(opts.blacklistCustomBlocks || []);
  delete opts.include;
  delete opts.exclude;
  var descriptors = new _WeakMap();

  function compileTemplate(id, _ref, source) {
    var functional = _ref.functional;

    var _compiler$compileToDe = compiler.compileToDescriptor(id.filename, "<template ".concat(functional ? 'functional' : '', ">\n").concat(source, "\n</template>")),
        template = _compiler$compileToDe.template;

    if (template.errors && template.errors.length) {
      console.error('> Errors: ' + path.relative(process.cwd(), id.filename) + '\n' + template.errors.map(function (it) {
        return '  - ' + it;
      }).join('\n'));
    }

    if (template.tips && template.tips.length) {
      console.log('> Tips: ' + path.relative(process.cwd(), id.filename) + '\n' + template.tips.map(function (it) {
        return '  - ' + it;
      }).join('\n'));
    }

    return "".concat(template.code, "\n export { render, staticRenderFns }");
  }

  return {
    name: 'vue.delegate',
    resolveId: function resolveId(id) {
      if (isVuePartRequest(id)) {
        var ref = parseVuePartRequest(id);
        var element = resolveVuePart(descriptors, ref);
        if (element.src) return path.resolve(path.dirname(ref.filename), element.src);
        return id;
      }
    },
    load: function load(id) {
      if (!isVuePartRequest(id)) return;
      id = parseVuePartRequest(id);
      return resolveVuePart(descriptors, id).content;
    },
    transform: function () {
      var _transform = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee(source, filename) {
        var descriptor, scopeId, input, result, id, element, _compiler$compileToDe2, styles;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!isVue(filename)) {
                  _context.next = 9;
                  break;
                }

                descriptor = descriptors[filename] = componentCompilerUtils.parse({
                  filename: filename,
                  source: source,
                  needMap: true
                });
                scopeId = 'data-v-' + (isProduction ? hash(path.basename(filename) + source) : hash(filename + source));
                input = {
                  scopeId: scopeId,
                  styles: [],
                  customBlocks: []
                };

                if (descriptor.template) {
                  input.template = {
                    code: "\n            import * as template from '".concat(createVuePartRequest(filename, descriptor.template.lang, 'template'), "'\n            var render = template.render\n            var staticRenderFns = template.staticRenderFns \n            "),
                    functional: 'functional' in descriptor.template.attrs
                  };
                }

                if (descriptor.script) {
                  input.script = {
                    code: "\n            export * from '".concat(createVuePartRequest(filename, descriptor.script.lang, 'script'), "'\n            import script from '").concat(createVuePartRequest(filename, descriptor.script.lang, 'script'), "'\n            export default script\n            ")
                  };
                }

                result = componentCompiler.assemble(compiler, filename, input, opts);
                descriptor.customBlocks.forEach(function (block, index) {
                  if (blacklisted.has(block.type)) return;
                  result.code += '\n' + "export * from '".concat(createVuePartRequest(filename, block.attrs.lang || createVuePartRequest.defaultLang[block.type] || block.type, 'customBlocks', index), "'");
                });
                return _context.abrupt("return", result);

              case 9:
                if (!isVuePartRequest(filename)) {
                  _context.next = 19;
                  break;
                }

                id = parseVuePartRequest(filename);
                element = resolveVuePart(descriptors, id);

                if (!(id.meta.type === 'styles')) {
                  _context.next = 17;
                  break;
                }

                _compiler$compileToDe2 = compiler.compileToDescriptor(id.filename, "<style ".concat(element.scoped ? 'scoped' : '', " ").concat(element.module ? 'module' + (typeof element.module === 'string' ? '="' + element.module + '"' : '') : '', ">\n").concat(source, "\n</style>")), styles = _compiler$compileToDe2.styles;
                return _context.abrupt("return", styles[0]);

              case 17:
                if (!(id.meta.type === 'template')) {
                  _context.next = 19;
                  break;
                }

                return _context.abrupt("return", compileTemplate(id, element, source));

              case 19:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function transform(_x, _x2) {
        return _transform.apply(this, arguments);
      };
    }()
  };
}

vue.delegate = vue$1;

module.exports = vue;
//# sourceMappingURL=rollup-plugin-vue.common.js.map
